package com.ibizlab.codegen.model;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.ibizlab.codegen.lite.DynamicModelStorage;
import com.ibizlab.codegen.lite.EntityModel;

import org.springframework.util.Assert;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;


public class ModelStorage {

    public static String USER_HOME = Paths.get(System.getProperty("user.home"),".ibizlab").toString();

    private DynamicModelStorage dynamicService=DynamicModelStorage.getInstance().modelPath(this.getModelPath());


    private String system=dynamicService.getDstSystemModel().getPssystemid();

    public String getModelPath()
    {
        if(modelPath.equals(File.separator))
            return modelPath.substring(0,modelPath.length()-1);
        return modelPath;
    }



    public DOModel loadDOModel(String entity)
    {
        EntityModel entityModel= null;
        try {
            entityModel = dynamicService.getDynamicEntity(entity);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
        Assert.notNull(entityModel,"loadDOModel未找到实体"+"."+entity);
        PojoSchema schema = TransUtils.EntityModelModel2Schema(entityModel);
        Assert.notNull(schema,"loadDOModel未找到实体"+"."+entity);
        DOModel doModel=new DOModel();
        doModel.setSchema(schema);
        for (String dsType : entityModel.getDsTypes()) {
            POSchema poSchema = TransUtils.EntityModelModel2PO(entityModel, dsType);
            if (poSchema != null) {
                doModel.addPOSchema(dsType, poSchema);
            }
        }
        return doModel;
    }


    public DOModel getDOModel(String entity)
    {
        DOModel doModel=null;
        PojoSchema schema=null;
        Path storePath = Paths.get(ModelStorage.USER_HOME,system,"repo",entity,"domain",entity+".json");
        if(!Files.exists(storePath))
        {
            try {
                String entityTag=dynamicService.getEntitiyIds().get(entity);
                Assert.hasLength(entityTag,"获取模型失败"+entity);
                if(!entityTag.endsWith("."+entity))
                {
                    String[] args=entityTag.split("[.]");
                    entity=args[2];
                    storePath = Paths.get(ModelStorage.USER_HOME,system,"repo",entity,"domain",entity+".json");
                }
            } catch (Exception exception) {
                throw new RuntimeException("获取模型失败"+entity,exception);
            }
        }

        File json= storePath.toFile();

        if((json.exists()&&json.lastModified()<dynamicService.getDstSystemModel().getLastModify())||(!json.exists()))
        {
            doModel = loadDOModel(entity);
            schema = doModel.getSchema();
            if(schema!=null)
            {
                if(doModel.getPoSchemas()!=null)
                {
                    Path poPath = Paths.get(ModelStorage.USER_HOME,system,"repo",doModel.getName(),"repository",doModel.getName()+".json");
                    try {
                        File dir=poPath.getParent().toFile();
                        if(!dir.exists())
                            dir.mkdirs();
                        Files.write(poPath, JSON.toJSONBytes(doModel.getPoSchemas()));
                    } catch (Exception e) {
                        throw new RuntimeException("保存文件失败POSchemas:"+poPath.toString());
                    }
                }
                schema.writeTo(Paths.get(ModelStorage.USER_HOME,system,"repo",doModel.getName(),"domain",doModel.getName()+".json"));
            }
        }
        else {
            doModel = new DOModel();
            schema = PojoSchema.fromPath(storePath);
            if(schema!=null) {
                doModel.setSchema(schema);
                Path poPath = Paths.get(ModelStorage.USER_HOME,  system,"repo", entity, "repository", entity + ".json");
                if (Files.exists(poPath)) {
                    try {
                        doModel.setPoSchemas(JSON.parseObject(new String(Files.readAllBytes(poPath), StandardCharsets.UTF_8), new TypeReference<LinkedHashMap<String, POSchema>>() {
                        }));
                    } catch (IOException e) {

                    }
                }
            }
        }
        Assert.notNull(schema,"未找到对应的模型"+entity);
        return doModel;
    }



}
