package com.ibizlab.codegen.model;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ibizlab.codegen.lite.*;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.LinkedHashMap;
import java.util.Map;

public class TransUtils {


    public static PojoSchema EntityModelModel2Schema(EntityModel entityModel)
    {
        PojoSchema pojoSchema=new PojoSchema().setName(entityModel.getEntityName()).setType("object").setTitle(entityModel.getLogicName()).setId(entityModel.getEntityId())
                .setOptions(new PojoOption().setDsTypes(String.join(",",entityModel.getDsTypes())).setLogicValid(entityModel.isLogicValid()).setLogicVal(entityModel.getLogicVal()).setLogicDelVal(entityModel.getLogicDelVal()));
        pojoSchema.getOptions().setAll(JSONObject.parseObject(JSON.toJSONString(entityModel.getEntity()))).remove("ext_params");


        int order=1;

        for(FieldModel fieldModel:entityModel.getFields())
        {
            String propType=fieldModel.getPropType();
            PojoSchema sub=null;
            if("date".equals(propType))
                sub=new PojoSchema().setType(PojoSchema.Type.string.getCode()).setFormat("date");
            else if("date-time".equals(propType))
                sub=new PojoSchema().setType(PojoSchema.Type.string.getCode()).setFormat("date-time");
            else
                sub=new PojoSchema().setType(propType);
            sub.setName(fieldModel.getCodeName()).setTitle(fieldModel.getField().getFieldLogicName()).setPropertyOrder(order)
                    .setOptions(new PojoOption().setAll(JSONObject.parseObject(JSON.toJSONString(fieldModel.getField()))));

            Object searchModes=fieldModel.getField().get("search_modes");
            if(!ObjectUtils.isEmpty(searchModes))
                sub.getOptions().setSearchModes(searchModes.toString());

            Object defaultValue=fieldModel.getField().get("default_value");
            if(!ObjectUtils.isEmpty(defaultValue))
                sub.getOptions().setDefaultValue(defaultValue.toString());

            sub.getOptions().remove("ext_params");


            if("PICKUP".equals(fieldModel.getField().getFieldType())&&fieldModel.isPhysicalField())
            {
                if(!StringUtils.isEmpty(sub.getOptions().getRelationCodeName()))
                {
                    RelationshipModel relationshipModel=entityModel.getRefMaps().get(sub.getOptions().getRelationCodeName());
                    if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getTableName())))
                        sub.getOptions().setRefTableName(relationshipModel.getTableName());
                }
                String smode=sub.getOptions().getSearchModes();

                if((!smode.startsWith("eq"))&&smode.indexOf(",eq")<0)
                    sub.getOptions().setSearchModes(smode.concat(StringUtils.isEmpty(smode)?"":",").concat("eq"));

            }

            pojoSchema.addProperty(sub.getName(),sub);
            order++;
        }

        if(entityModel.getReferences()!=null)
        {
            for(RelationshipModel rel:entityModel.getReferences())
            {
                PojoSchema sub=new PojoSchema().setType(PojoSchema.Type.object.getCode()).setName(rel.getCodeName()).setTitle(rel.getCodeName()+rel.getEntityLogicName())
                        .setPropertyOrder(order).setOptions(new PojoOption()
                        .set("logic_name",rel.getEntityLogicName())
                                .set("entity_name",rel.getEntityName())
                                .set("system_id",(StringUtils.isEmpty(rel.getSystemId())?entityModel.getSystemId():rel.getSystemId()))
                                .set("entity_id",rel.getEntityId()).set("code_name",rel.getEntityCodeName())
                                .set("table_name",rel.getTableName()).set("ds_name",rel.getDataSourceName())
                                        .set("relation_name",rel.getRelation().getName())
                                        .set("relation",JSONObject.parseObject(JSON.toJSONString(rel.getRelation()), MetaRelationshipModel.class))

                        )
                        .setRef("domain."+rel.getEntityName());
                pojoSchema.addProperty(sub.getName(),sub);
                order++;
            }
        }


        if(entityModel.getNesteds()!=null)
        {
            for(RelationshipModel nest:entityModel.getNesteds())
            {
                PojoSchema sub=new PojoSchema().setType(PojoSchema.Type.array.getCode()).setName(nest.getCodeName()).setTitle(nest.getCodeName()+nest.getEntityLogicName())
                        .setPropertyOrder(order).setOptions(new PojoOption()
                                .set("logic_name",nest.getEntityLogicName())
                                .set("entity_name",nest.getEntityName())
                                .set("system_id",(StringUtils.isEmpty(nest.getSystemId())?entityModel.getSystemId():nest.getSystemId()))
                                .set("entity_id",nest.getEntityId()).set("code_name",nest.getEntityCodeName())
                                .set("table_name",nest.getTableName()).set("ds_name",nest.getDataSourceName())
                                .set("relation_name",nest.getRelation().getName())
                                .set("relation",JSONObject.parseObject(JSON.toJSONString(nest.getRelation()), MetaRelationshipModel.class))
                        )
                        .setItems(new PojoSchema().setType(PojoSchema.Type.object.getCode()).setName(nest.getEntityName()).setTitle(nest.getEntityLogicName())
                                .setRef("domain."+nest.getEntityName())
                                .setOptions(new PojoOption()
                                        .set("logic_name",nest.getEntityLogicName())
                                        .set("entity_name",nest.getEntityName())
                                        .set("system_id",(StringUtils.isEmpty(nest.getSystemId())?entityModel.getSystemId():nest.getSystemId()))
                                        .set("entity_id",nest.getEntityId()).set("code_name",nest.getEntityCodeName())
                                        .set("table_name",nest.getTableName()).set("ds_name",nest.getDataSourceName())
                                )
                        );
                pojoSchema.addProperty(sub.getName(),sub);
                order++;
            }
        }




        return pojoSchema;
    }

    public static POSchema PojoSchema2PO(PojoSchema pojoSchema)
    {
        POSchema poSchema=new POSchema().setName(pojoSchema.getDefaultTableName()).setDefaultDataSource(pojoSchema.getDefaultDataSoruce())
                .setRemarks(pojoSchema.getTitle()).setLogicVal(pojoSchema.getOptions().getLogicVal()).setLogicDelVal(pojoSchema.getOptions().getLogicDelVal());


        for(String name:pojoSchema.getProperties().keySet())
        {
            PojoSchema sub=pojoSchema.getProperties().get(name);


            String dataType=sub.getOptions().getDataType();
            Integer length=sub.getOptions().getDataLength();
            Integer precision=sub.getOptions().getDataPreci();

            POSchema.Column column=new POSchema.Column().setName(sub.getOptions().getFieldName()).setAlias(name).setRemarks(sub.getTitle()).setDefaultValue(sub.getOptions().getDefaultValue()).setPredefined(sub.getOptions().getPredefined()).setLength(length);

            if(PojoSchema.Type.object.getCode().equals(sub.getType())||PojoSchema.Type.array.getCode().equals(sub.getType()))
            {
                if(!sub.getOptions().isPhysicalField())
                    continue;

                if(StringUtils.isEmpty(dataType))dataType="TEXT";
                column.setType(dataType);
                column.setLength(null);
            }
            else if(PojoSchema.Type.string.getCode().equals(sub.getType())&&(!StringUtils.isEmpty(sub.getFormat())))
            {
                if(StringUtils.isEmpty(dataType)&&(sub.getFormat().indexOf("date")>=0))
                    dataType="DATETIME";
                else
                    dataType="VARCHAR";
                column.setType(dataType);
            }
            else if(PojoSchema.Type.integer.getCode().equals(sub.getType()) )
            {
                if(StringUtils.isEmpty(dataType))dataType="INT";
                column.setType(dataType);
            }
            else if(PojoSchema.Type.number.getCode().equals(sub.getType()) )
            {
                if(StringUtils.isEmpty(dataType))dataType="NUMBER";
                column.setType(dataType);
            }
            else {
                if(StringUtils.isEmpty(dataType))dataType="VARCHAR";
                column.setType(dataType);
            }

            if(column.isNumber())
                column.setPrecision(precision);

            if((pojoSchema.getRequired()!=null&&pojoSchema.getRequired().contains(name))||(!pojoSchema.getOptions().isNullable()))
                column.getConstraints(true).setNullable(false);

            if(pojoSchema.getKeyMap()!=null&&pojoSchema.getKeyMap().containsKey(column.getName())) {
                String primaryKeyName="PK_"+poSchema.getName().toUpperCase()+"_"+ column.getName().toUpperCase();
                column.getConstraints(true).setPrimaryKey(true).setPrimaryKeyName(primaryKeyName);
            }
            if(sub.getOptions().isLogicValidField())
            {
                if(!StringUtils.isEmpty(pojoSchema.getOptions().getLogicVal()))
                    column.setDefaultValue(pojoSchema.getOptions().getLogicVal());
                else
                    column.setDefaultValue("1");
            }
            if(!StringUtils.isEmpty(sub.getOptions().getSearchModes()))
                column.putSearchModes(sub.getOptions().getSearchModes());

            if("PICKUP".equals(sub.getOptions().getFieldType())&&sub.getOptions().isPhysicalField())
            {
                PojoSchema relationshipModel=pojoSchema.getRefSchema(sub.getOptions().getRelationName());
                String fkName=sub.getOptions().getRelationName().toUpperCase();
                if((!StringUtils.isEmpty(relationshipModel.getOptions().getRefTableName()))&&(!StringUtils.isEmpty(fkName))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getOptions().getRefTableName()).setReferencedColumnNames(sub.getOptions().getRefFieldName()).setForeignKeyName(fkName);
                    poSchema.addForeignKeyConstraint(new POSchema.ForeignKeyConstraint().setConstraintName(fkName).setBaseTableName(poSchema.getName()).setBaseColumnNames(column.getName()).setReferencedTableName(relationshipModel.getOptions().getRefTableName()).setReferencedColumnNames(sub.getOptions().getRefFieldName()));
                }
            }
            else  if((!StringUtils.isEmpty(sub.getOptions().getRelationName()))) {
                PojoSchema relationshipModel=pojoSchema.getRefSchema(sub.getOptions().getRelationName());
                if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getOptions().getTableName()))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getOptions().getTableName())
                            .setReferencedColumnNames(sub.getOptions().getRefFieldName());
                }
            }

            if(!sub.getOptions().isPhysicalField())
                poSchema.addTransient(column.setComputed(true));
            else
                poSchema.addColumn(column);

        }
        String defaultQueryScript=pojoSchema.getOptions().getDefaultQueryScript();
        if(!StringUtils.isEmpty(defaultQueryScript))
            poSchema.setDefaultQueryScriptSQL(defaultQueryScript);

        return poSchema;
    }



    public static POSchema PojoSchema2DocumentPO(PojoSchema pojoSchema)
    {
        POSchema poSchema=new POSchema().setName(pojoSchema.getCodeName().toLowerCase()).setDefaultDataSource(pojoSchema.getDefaultDataSoruce())
                .setRemarks(pojoSchema.getTitle()).setLogicVal(pojoSchema.getOptions().getLogicVal()).setLogicDelVal(pojoSchema.getOptions().getLogicDelVal());


        for(String name:pojoSchema.getProperties().keySet())
        {
            PojoSchema sub=pojoSchema.getProperties().get(name);


            String dataType=sub.getOptions().getDataType();
            Integer length=sub.getOptions().getDataLength();
            Integer precision=sub.getOptions().getDataPreci();

            POSchema.Column column=new POSchema.Column().setName(name).setAlias(name).setRemarks(sub.getTitle()).setDefaultValue(sub.getOptions().getDefaultValue()).setPredefined(sub.getOptions().getPredefined()).setLength(length);

            if(PojoSchema.Type.object.getCode().equals(sub.getType())||PojoSchema.Type.array.getCode().equals(sub.getType()))
            {
                if(!sub.getOptions().isPhysicalField())
                    continue;

                if(StringUtils.isEmpty(dataType))dataType="TEXT";
                column.setType(dataType);
                column.setLength(null);
            }
            else if(PojoSchema.Type.string.getCode().equals(sub.getType())&&(!StringUtils.isEmpty(sub.getFormat())))
            {
                if(StringUtils.isEmpty(dataType)&&(sub.getFormat().indexOf("date")>=0))
                    dataType="DATETIME";
                else
                    dataType="VARCHAR";
                column.setType(dataType);
            }
            else if(PojoSchema.Type.integer.getCode().equals(sub.getType()) )
            {
                if(StringUtils.isEmpty(dataType))dataType="INT";
                column.setType(dataType);
            }
            else if(PojoSchema.Type.number.getCode().equals(sub.getType()) )
            {
                if(StringUtils.isEmpty(dataType))dataType="NUMBER";
                column.setType(dataType);
            }
            else {
                if(StringUtils.isEmpty(dataType))dataType="VARCHAR";
                column.setType(dataType);
            }

            if(column.isNumber())
                column.setPrecision(precision);

            if((pojoSchema.getRequired()!=null&&pojoSchema.getRequired().contains(name))||(!pojoSchema.getOptions().isNullable()))
                column.getConstraints(true).setNullable(false);

            if(pojoSchema.getKeyMap()!=null&&pojoSchema.getKeyMap().containsKey(column.getName())) {
                String primaryKeyName="PK_"+poSchema.getName().toUpperCase()+"_"+ column.getName().toUpperCase();
                column.getConstraints(true).setPrimaryKey(true).setPrimaryKeyName(primaryKeyName);
            }
            if(sub.getOptions().isLogicValidField())
            {
                if(!StringUtils.isEmpty(pojoSchema.getOptions().getLogicVal()))
                    column.setDefaultValue(pojoSchema.getOptions().getLogicVal());
                else
                    column.setDefaultValue("1");
            }
            if(!StringUtils.isEmpty(sub.getOptions().getSearchModes()))
                column.putSearchModes(sub.getOptions().getSearchModes());

            if("PICKUP".equals(sub.getOptions().getFieldType())&&sub.getOptions().isPhysicalField())
            {
                PojoSchema relationshipModel=pojoSchema.getRefSchema(sub.getOptions().getRelationName());
                String fkName=sub.getOptions().getRelationName().toUpperCase();
                if((!StringUtils.isEmpty(relationshipModel.getOptions().getRefTableName()))&&(!StringUtils.isEmpty(fkName))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getOptions().getRefTableName()).setReferencedColumnNames(sub.getOptions().getRefFieldName()).setForeignKeyName(fkName);
                    poSchema.addForeignKeyConstraint(new POSchema.ForeignKeyConstraint().setConstraintName(fkName).setBaseTableName(poSchema.getName()).setBaseColumnNames(column.getName()).setReferencedTableName(relationshipModel.getOptions().getRefTableName()).setReferencedColumnNames(sub.getOptions().getRefFieldName()));
                }
            }
            else  if((!StringUtils.isEmpty(sub.getOptions().getRelationName()))) {
                PojoSchema relationshipModel=pojoSchema.getRefSchema(sub.getOptions().getRelationName());
                if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getOptions().getTableName()))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getOptions().getTableName())
                            .setReferencedColumnNames(sub.getOptions().getRefFieldName());
                }
            }

            if(!sub.getOptions().isPhysicalField())
                poSchema.addTransient(column.setComputed(true));
            else
                poSchema.addColumn(column);

        }
        String defaultQueryScript=pojoSchema.getOptions().getDefaultQueryScript();
        if(!StringUtils.isEmpty(defaultQueryScript))
            poSchema.setDefaultQueryScriptSQL(defaultQueryScript);

        return poSchema;
    }


    public static POSchema EntityModelModel2PO(EntityModel entityModel, String dsType) {

        final String dataSourceType=dsType.toLowerCase();
        POSchema poSchema=new POSchema().setDsType(dataSourceType).setName(entityModel.getTableName(dataSourceType)).setDefaultDataSource(entityModel.getDsName())
                .setRemarks(entityModel.getLogicName()).setLogicVal(entityModel.getLogicVal()).setLogicDelVal(entityModel.getLogicDelVal());


        Map<String,FieldModel> keyMap=new LinkedHashMap<>();
        Map<String,RelationshipModel> relationshipModelMap = new LinkedHashMap<>();
        if(entityModel.getKeyFields()!=null)entityModel.getKeyFields().forEach(model->keyMap.put(model.getColumnName(),model));
        if(entityModel.getReferences()!=null)entityModel.getReferences().forEach(model->relationshipModelMap.put(model.getRelation().getName(),model));

        int order = 1;
        for (FieldModel fieldModel : entityModel.getFields()) {
            MetaFieldModel sub=fieldModel.getField();

            String colName=fieldModel.getColumnName(dataSourceType);

            Integer length=sub.getDataLength();
            Integer precision=sub.getDataPreci();

            POSchema.Column column=new POSchema.Column().setName(colName).setAlias(sub.getCodeName()).setRemarks(sub.getFieldLogicName()).setPredefined(sub.getPredefined()).setLength(length).setType(sub.getDataType());

            if(column.isNumber())
                column.setPrecision(precision);

            if(sub.getNullable()!=null&&sub.getNullable()==0)
                column.getConstraints(true).setNullable(false);

            if(keyMap.containsKey(column.getName())) {
                String primaryKeyName="PK_"+poSchema.getName().toUpperCase()+"_"+ column.getName().toUpperCase();
                column.getConstraints(true).setPrimaryKey(true).setPrimaryKeyName(primaryKeyName);
            }
            if(fieldModel.isLogicValidField())
            {
                if(!StringUtils.isEmpty(entityModel.getLogicVal()))
                    column.setDefaultValue(entityModel.getLogicVal());
                else
                    column.setDefaultValue("1");
            }

            Object searchModes=fieldModel.getField().get("search_modes");
            if(searchModes!=null)
                column.putSearchModes(searchModes.toString());

            if("PICKUP".equals(sub.getFieldType())&&fieldModel.isPhysicalField())
            {
                RelationshipModel relationshipModel=relationshipModelMap.get(sub.getRelationName());
                String fkName=sub.getRelationName().toUpperCase();
                if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getTableName()))&&(!StringUtils.isEmpty(fkName))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getTableName())
                            .setReferencedColumnNames(sub.getRefFieldName()).setForeignKeyName(fkName);

                    poSchema.addForeignKeyConstraint(new POSchema.ForeignKeyConstraint().setConstraintName(fkName)
                            .setBaseTableName(poSchema.getName()).setBaseColumnNames(column.getName())
                            .setReferencedTableName(relationshipModel.getTableName()).setReferencedColumnNames(sub.getRefFieldName()));
                }
                if(!column.getSearchModes().contains("eq"))
                    column.getSearchModes().add("eq");

            }
            else  if((!StringUtils.isEmpty(sub.getRelationName()))) {
                RelationshipModel relationshipModel = relationshipModelMap.get(sub.getRelationName());
                String fkName=sub.getRelationName().toUpperCase();
                if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getTableName()))&&(!StringUtils.isEmpty(fkName))) {
                    column.getConstraints(true).setReferencedTableName(relationshipModel.getTableName())
                            .setReferencedColumnNames(sub.getRefFieldName());
                }
            }

            if(!fieldModel.isPhysicalField())
                poSchema.addTransient(column.setComputed(true));
            else
                poSchema.addColumn(column);

            order++;
        }

        if(entityModel.getDataSets()!=null)
        {

            entityModel.getDataSets().forEach(dataSet->{

                if(dataSet.getDatasetId().startsWith(entityModel.getEntityName().toLowerCase()+"-")&&dataSet.getDatasetId().endsWith(dataSourceType))
                {
                    POSchema.Segment segment=new POSchema.Segment().setName(dataSet.getDatasetId()).setVendorProvider(dataSourceType).setBody(dataSet.getDsCode());
                    if(!StringUtils.isEmpty(dataSet.getDsModel()))
                        segment.setFormat(dataSet.getDsModel());
                    poSchema.addSegment(segment);
                }
                else  if(!dataSet.getDatasetId().startsWith(entityModel.getEntityName().toLowerCase()+"-"))
                {
                    POSchema.Segment segment=new POSchema.Segment().setName(dataSet.getCodeName()).setVendorProvider(dataSourceType).setBody(dataSet.getDsCode());
                    if(!StringUtils.isEmpty(dataSet.getDsModel()))
                        segment.setFormat(dataSet.getDsModel());
                    poSchema.addSegment(segment);
                }
            });

        }


        return poSchema;
    }


}
